# $Id: dce_rpc.pac,v 1.2 2005/09/07 17:34:57 vern Exp $
#
# Definitions for DCE RPC.

enum dce_rpc_ptype {
	DCE_RPC_REQUEST,
	DCE_RPC_PING,
	DCE_RPC_RESPONSE,
	DCE_RPC_FAULT,
	DCE_RPC_WORKING,
	DCE_RPC_NOCALL,
	DCE_RPC_REJECT,
	DCE_RPC_ACK,
	DCE_RPC_CL_CANCEL,
	DCE_RPC_FACK,
	DCE_RPC_CANCEL_ACK,
	DCE_RPC_BIND,
	DCE_RPC_BIND_ACK,
	DCE_RPC_BIND_NAK,
	DCE_RPC_ALTER_CONTEXT,
	DCE_RPC_ALTER_CONTEXT_RESP,
	DCE_RPC_SHUTDOWN,
	DCE_RPC_CO_CANCEL,
	DCE_RPC_ORPHANED,
};

type uuid = uint8[16];

type context_handle = record {
	cxt_attributes:		uint32;
	cxt_uuid:		uuid;
};

type rpc_if_id_t = record {
	if_uuid		: uuid;
	vers_major	: uint16;
	vers_minor	: uint16;
};

# typedef [ptr] rpc_if_id_t *rpc_if_id_p_t;

type NDR_Format = record {
	intchar		: uint8;
	floatspec	: uint8;
	reserved	: padding[2];

	let byteorder = (intchar >> 4) ? littleendian : bigendian;
};

type DCE_RPC_Header = record {
	rpc_vers	: uint8 &check(rpc_vers == 5);
	rpc_vers_minor	: uint8;
	PTYPE		: uint8;
	pfc_flags	: uint8;
	packed_drep	: NDR_Format;
	frag_length	: uint16;
	auth_length	: uint16;
	call_id		: uint32;
} &byteorder = packed_drep.byteorder;

type p_context_id_t = uint16;

type p_syntax_id_t = record {
	if_uuid		: uuid;
	if_version	: uint32;
};

type p_cont_elem_t = record {
	p_cont_id	: p_context_id_t;
	n_transfer_syn	: uint8;
	reserved	: padding[1];
	abstract_syntax	: p_syntax_id_t;
	transfer_syntaxes : p_syntax_id_t[n_transfer_syn];
};

type p_cont_list_t = record {
	n_context_elem	: uint8;
	reserved	: padding[3];
	p_cont_elem	: p_cont_elem_t[n_context_elem];
};

type DCE_RPC_Bind = record {
	max_xmit_frag	: uint16;
	max_recv_frag	: uint16;
	assoc_group_id	: uint32;
	p_context_elem	: p_cont_list_t;
};

type DCE_RPC_Request(length) = record {
	alloc_hint	: uint32;
	p_cont_id	: p_context_id_t;
	opnum		: uint16;
	# object		: uuid;
	# stub_pad_0	: padding align 8;
	stub		: uint8[length - offsetof(stub)];
};

type DCE_RPC_Response(length) = record {
	alloc_hint	: uint32;
	p_cont_id	: p_context_id_t;
	cancel_count	: uint8;
	reserved	: uint8;
	# stub_pad_0	: padding align 8;
	stub		: uint8[length - offsetof(stub)];
};

type DCE_RPC_Body(header: DCE_RPC_Header, length) = case header.PTYPE of {
	DCE_RPC_BIND 	 => bind	: DCE_RPC_Bind;
	DCE_RPC_REQUEST	 => request 	: DCE_RPC_Request(length);
	DCE_RPC_RESPONSE => response	: DCE_RPC_Response(length);
	default		 => other	: uint8[length];
};

type DCE_RPC_Auth(header: DCE_RPC_Header) = case header.auth_length of {
	0		=> none			: empty;
	default		=> auth_verifier	: uint8[header.auth_length];
};

type DCE_RPC_PDU = record {
	let body_length =
		header.frag_length - sizeof(header) - header.auth_length;

	# Set header's byteorder to littleendian (or bigendian) to
	# avoid cyclic dependency
	header	: DCE_RPC_Header &byteorder = littleendian;
	body	: DCE_RPC_Body(header, body_length);
	auth	: DCE_RPC_Auth(header);
} &byteorder = header.byteorder;

type epmapper_lookup_req = record {
	inquiry_type	: uint32;
	# object	: uuid_p_t;
	object		: uint32;
	# interface_id	: rpc_if_id_p_t;
	interface_id	: uint32;
	vers_option	: uint32;
	entry_handle	: context_handle;
};

type epmapper_map_req = record {
};

type epm_uuid 	= record {
	if_uuid		: uuid;
	if_version	: uint16;
} &byteorder = littleendian;

type epm_port 	= uint16 &byteorder = bigendian;
type epm_ip 	= uint32 &byteorder = bigendian;

enum epm_protocol {
	EPM_PROTOCOL_TCP	= 0x07,
	EPM_PROTOCOL_UDP	= 0x08,
	EPM_PROTOCOL_IP		= 0x09,
	EPM_PROTOCOL_UUID	= 0x0d,
};

type epm_lhs_data(length, protocol) = case protocol of {
	EPM_PROTOCOL_UUID 	=>	uuid : epm_uuid;
	default			=> 	other: uint8[length];
};

type epm_rhs_data(length, protocol) = case protocol of {
	EPM_PROTOCOL_TCP	=>	tcp: 	epm_port;
	EPM_PROTOCOL_UDP	=>	udp: 	epm_port;
	EPM_PROTOCOL_IP		=>	ip: 	epm_ip;
	default			=> 	other: 	uint8[length];
};

type epm_lhs = record {
	length		: uint16;
	protocol	: uint8;
	data		: epm_lhs_data(length - 1, protocol);
} &byteorder = littleendian;

type epm_rhs(protocol) = record {
	length		: uint16;
	data		: epm_rhs_data(length, protocol);
} &byteorder = littleendian;

type epm_floor = record {
	let protocol = lhs.protocol;
	lhs		: epm_lhs;
	rhs		: epm_rhs(protocol);
};

type epm_tower = record {
	num_floors	: uint16;
	floors		: epm_floor[num_floors];
} &byteorder = littleendian;

type epm_twr_p = record {
	# Where are there length and tower_length?
	ref		: uint32;
	length		: uint32;
	tower_length	: uint32;
	tower		: epm_tower &length = tower_length;
};

type epm_tower_array = record {
	# A lot of questions here ...
	# Why does the array include max_count, offset, and actual_count?
	# Is it because it's a "conformant varying array"?
	# How long should the towers be? actual_count or offset + actual_count?

	max_count	: uint32;
	offset		: uint32;
	actual_count	: uint32;
	towers		: epm_twr_p[actual_count];
};

type epmapper_map_resp = record {
	entry_handle	: context_handle;
	num_towers	: uint32;
	towers		: epm_tower_array;
	return_code	: uint32;
};
